import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';
import { timer } from 'rxjs/observable/timer';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { ScheduleService } from './schedule.service';

import { Event } from '../models/event';


export class ScheduleDataSource implements DataSource<Event> {
    private scheduleSubject = new BehaviorSubject<Event[]>([]);
    private loadingSubject = new BehaviorSubject<boolean>(false);
    private countdownSubject = new BehaviorSubject<number>(undefined);

    public loading$ = this.loadingSubject.asObservable();
    public countdown$ = this.countdownSubject.asObservable();

    private lastCountdown = 0;

    constructor(private scheduleService: ScheduleService) { }

    loadSchedule() {
        this.loadingSubject.next(true);

        this.scheduleService.findEvents()
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe((schedule: Event[]) => {
                this.scheduleSubject.next(schedule);

                timer(0, 1000).subscribe(() => {
                    if (schedule.length == 0) {
                        return;
                    }

                    let nearest = new Date(schedule[0].start + 'Z');
                    let countdown = nearest.getTime() - Date.now();
                    let h = countdown - this.lastCountdown;

                    if (h > 0 && h < 2000) {
                        countdown = this.lastCountdown;
                    }

                    this.lastCountdown = countdown;

                    this.countdownSubject.next(countdown);
                });
            });
    }

    connect(collectionViewer: CollectionViewer): Observable<Event[]> {
        console.log('Connecting data source');
        return this.scheduleSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.scheduleSubject.complete();
        this.loadingSubject.complete();
        this.countdownSubject.complete();
    }
}
